/* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
 * Copyright 1998-2003 Decklin Foster <decklin@red-bean.com>
 * This program is free software; see LICENSE for details. */

#ifndef AEWM_CLIENTS_XAW_UTIL_H
#define AEWM_CLIENTS_XAW_UTIL_H

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>

void popup_cb(Widget, XtPointer, XtPointer);

#endif /* AEWM_CLIENTS_XAW_UTIL_H */
